use alloc::{collections::BTreeMap, string::String, vec::Vec, string::ToString, format};
use core::fmt;

#[derive(Debug, Clone, Copy, PartialEq)]
pub enum FileType {
    Regular,
    Directory,
    Executable,
}

#[derive(Debug, Clone)]
pub struct FileMetadata {
    pub name: String,
    pub size: usize,
    pub file_type: FileType,
    pub permissions: u32,
}

#[derive(Debug, Clone)]
pub struct File {
    pub metadata: FileMetadata,
    pub data: Vec<u8>,
}

#[derive(Debug, Clone)]
pub struct Directory {
    pub metadata: FileMetadata,
    pub entries: BTreeMap<String, FileSystemEntry>,
}

#[derive(Debug, Clone)]
pub enum FileSystemEntry {
    File(File),
    Directory(Directory),
}

pub struct MemoryFileSystem {
    root: Directory,
    current_dir: String,
}

impl MemoryFileSystem {
    pub fn new() -> Self {
        let root_metadata = FileMetadata {
            name: "/".to_string(),
            size: 0,
            file_type: FileType::Directory,
            permissions: 0o755,
        };

        let mut root = Directory {
            metadata: root_metadata,
            entries: BTreeMap::new(),
        };

        // Create initial directories
        root.entries.insert("bin".to_string(), FileSystemEntry::Directory(Directory {
            metadata: FileMetadata {
                name: "bin".to_string(),
                size: 0,
                file_type: FileType::Directory,
                permissions: 0o755,
            },
            entries: BTreeMap::new(),
        }));

        root.entries.insert("home".to_string(), FileSystemEntry::Directory(Directory {
            metadata: FileMetadata {
                name: "home".to_string(),
                size: 0,
                file_type: FileType::Directory,
                permissions: 0o755,
            },
            entries: BTreeMap::new(),
        }));

        root.entries.insert("tmp".to_string(), FileSystemEntry::Directory(Directory {
            metadata: FileMetadata {
                name: "tmp".to_string(),
                size: 0,
                file_type: FileType::Directory,
                permissions: 0o777,
            },
            entries: BTreeMap::new(),
        }));

        Self {
            root,
            current_dir: "/".to_string(),
        }
    }

    pub fn create_file(&mut self, path: &str, content: &[u8]) -> Result<(), &'static str> {
        let (dir_path, filename) = self.split_path(path);
        let directory = self.get_directory_mut(&dir_path)?;

        let file = File {
            metadata: FileMetadata {
                name: filename.clone(),
                size: content.len(),
                file_type: FileType::Regular,
                permissions: 0o644,
            },
            data: content.to_vec(),
        };

        directory.entries.insert(filename, FileSystemEntry::File(file));
        Ok(())
    }

    pub fn create_executable(&mut self, path: &str, content: &[u8]) -> Result<(), &'static str> {
        let (dir_path, filename) = self.split_path(path);
        let directory = self.get_directory_mut(&dir_path)?;

        let file = File {
            metadata: FileMetadata {
                name: filename.clone(),
                size: content.len(),
                file_type: FileType::Executable,
                permissions: 0o755,
            },
            data: content.to_vec(),
        };

        directory.entries.insert(filename, FileSystemEntry::File(file));
        Ok(())
    }

    pub fn read_file(&self, path: &str) -> Result<&Vec<u8>, &'static str> {
        let (dir_path, filename) = self.split_path(path);
        let directory = self.get_directory(&dir_path)?;

        match directory.entries.get(&filename) {
            Some(FileSystemEntry::File(file)) => Ok(&file.data),
            Some(FileSystemEntry::Directory(_)) => Err("Path is a directory"),
            None => Err("File not found"),
        }
    }

    pub fn create_directory(&mut self, path: &str) -> Result<(), &'static str> {
        let (parent_path, dirname) = self.split_path(path);
        let parent = self.get_directory_mut(&parent_path)?;

        let directory = Directory {
            metadata: FileMetadata {
                name: dirname.clone(),
                size: 0,
                file_type: FileType::Directory,
                permissions: 0o755,
            },
            entries: BTreeMap::new(),
        };

        parent.entries.insert(dirname, FileSystemEntry::Directory(directory));
        Ok(())
    }

    pub fn list_directory(&self, path: &str) -> Result<Vec<&FileMetadata>, &'static str> {
        let directory = self.get_directory(path)?;
        let mut entries = Vec::new();

        for entry in directory.entries.values() {
            match entry {
                FileSystemEntry::File(file) => entries.push(&file.metadata),
                FileSystemEntry::Directory(dir) => entries.push(&dir.metadata),
            }
        }

        Ok(entries)
    }

    pub fn remove_file(&mut self, path: &str) -> Result<(), &'static str> {
        let (dir_path, filename) = self.split_path(path);
        let directory = self.get_directory_mut(&dir_path)?;

        match directory.entries.remove(&filename) {
            Some(FileSystemEntry::File(_)) => Ok(()),
            Some(FileSystemEntry::Directory(_)) => {
                directory.entries.insert(filename.clone(), 
                    FileSystemEntry::Directory(Directory {
                        metadata: FileMetadata {
                            name: filename.clone(),
                            size: 0,
                            file_type: FileType::Directory,
                            permissions: 0o755,
                        },
                        entries: BTreeMap::new(),
                    }));
                Err("Cannot remove directory with rm")
            },
            None => Err("File not found"),
        }
    }

    pub fn change_directory(&mut self, path: &str) -> Result<(), &'static str> {
        let absolute_path = if path.starts_with('/') {
            path.to_string()
        } else {
            if self.current_dir == "/" {
                format!("/{}", path)
            } else {
                format!("{}/{}", self.current_dir, path)
            }
        };

        // Check if directory exists
        self.get_directory(&absolute_path)?;
        self.current_dir = absolute_path;
        Ok(())
    }

    pub fn get_current_directory(&self) -> &str {
        &self.current_dir
    }

    fn split_path(&self, path: &str) -> (String, String) {
        if path == "/" {
            return ("/".to_string(), "".to_string());
        }

        let full_path = if path.starts_with('/') {
            path.to_string()
        } else {
            if self.current_dir == "/" {
                format!("/{}", path)
            } else {
                format!("{}/{}", self.current_dir, path)
            }
        };

        if let Some(last_slash) = full_path.rfind('/') {
            if last_slash == 0 {
                ("/".to_string(), full_path[1..].to_string())
            } else {
                (full_path[..last_slash].to_string(), full_path[last_slash + 1..].to_string())
            }
        } else {
            (self.current_dir.clone(), path.to_string())
        }
    }

    fn get_directory(&self, path: &str) -> Result<&Directory, &'static str> {
        if path == "/" {
            return Ok(&self.root);
        }

        let mut current = &self.root;
        let parts: Vec<&str> = path.trim_start_matches('/').split('/').filter(|s| !s.is_empty()).collect();

        for part in parts {
            match current.entries.get(part) {
                Some(FileSystemEntry::Directory(dir)) => current = dir,
                Some(FileSystemEntry::File(_)) => return Err("Path component is a file"),
                None => return Err("Directory not found"),
            }
        }

        Ok(current)
    }

    fn get_directory_mut(&mut self, path: &str) -> Result<&mut Directory, &'static str> {
        if path == "/" {
            return Ok(&mut self.root);
        }

        let mut current = &mut self.root;
        let parts: Vec<&str> = path.trim_start_matches('/').split('/').filter(|s| !s.is_empty()).collect();

        for part in parts {
            match current.entries.get_mut(part) {
                Some(FileSystemEntry::Directory(dir)) => current = dir,
                Some(FileSystemEntry::File(_)) => return Err("Path component is a file"),
                None => return Err("Directory not found"),
            }
        }

        Ok(current)
    }
}

impl fmt::Display for FileType {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self {
            FileType::Regular => write!(f, "-"),
            FileType::Directory => write!(f, "d"),
            FileType::Executable => write!(f, "x"),
        }
    }
}