use alloc::{collections::BTreeMap, string::String, vec::Vec, string::ToString, format, vec};
use core::fmt;
use crate::filesystem::MemoryFileSystem;
use crate::systemd::SystemManager;

#[derive(Debug, Clone)]
pub struct PackageMetadata {
    pub name: String,
    pub version: String,
    pub description: String,
    pub arch: String,
    pub size: usize,
    pub dependencies: Vec<String>,
}

#[derive(Debug, Clone)]
pub struct PackageFile {
    pub path: String,
    pub content: Vec<u8>,
    pub executable: bool,
}

#[derive(Debug, Clone)]
pub struct ServiceConfig {
    pub name: String,
    pub command: String,
    pub container: bool,
    pub auto_start: bool,
}

#[derive(Debug, Clone)]
pub struct Package {
    pub metadata: PackageMetadata,
    pub files: Vec<PackageFile>,
    pub services: Vec<ServiceConfig>,
}

#[derive(Debug)]
pub struct Repository {
    pub name: String,
    pub url: String,
    pub packages: BTreeMap<String, PackageMetadata>,
}

pub struct PackageManager {
    installed_packages: BTreeMap<String, Package>,
    repositories: Vec<Repository>,
    package_db_path: String,
}

impl PackageManager {
    pub fn new() -> Self {
        let mut pm = Self {
            installed_packages: BTreeMap::new(),
            repositories: Vec::new(),
            package_db_path: "/var/pkg".to_string(),
        };

        // Add default repository
        pm.add_repository("aios-main".to_string(), "https://packages.aios.dev".to_string());
        
        // Create some built-in packages
        pm.create_builtin_packages();
        
        pm
    }

    fn create_builtin_packages(&mut self) {
        // Claude Code package
        let claude_package = Package {
            metadata: PackageMetadata {
                name: "claude-code".to_string(),
                version: "1.0.0".to_string(),
                description: "AI Development Environment".to_string(),
                arch: "x86_64".to_string(),
                size: 1024 * 1024, // 1MB
                dependencies: vec!["nodejs".to_string(), "python".to_string()],
            },
            files: vec![
                PackageFile {
                    path: "/bin/claude-code".to_string(),
                    content: b"AIOS\x00\x10\x00\x00claude-code-ai-assistant".to_vec(),
                    executable: true,
                },
                PackageFile {
                    path: "/lib/claude/runtime.so".to_string(),
                    content: b"Claude Runtime Library".to_vec(),
                    executable: false,
                },
            ],
            services: vec![
                ServiceConfig {
                    name: "claude-code".to_string(),
                    command: "/bin/claude-code --daemon".to_string(),
                    container: true,
                    auto_start: false,
                },
            ],
        };

        // Node.js package
        let nodejs_package = Package {
            metadata: PackageMetadata {
                name: "nodejs".to_string(),
                version: "18.0.0".to_string(),
                description: "JavaScript Runtime".to_string(),
                arch: "x86_64".to_string(),
                size: 2 * 1024 * 1024, // 2MB
                dependencies: vec![],
            },
            files: vec![
                PackageFile {
                    path: "/bin/node".to_string(),
                    content: b"AIOS\x00\x10\x00\x00nodejs-runtime".to_vec(),
                    executable: true,
                },
                PackageFile {
                    path: "/bin/npm".to_string(),
                    content: b"AIOS\x00\x10\x00\x00npm-package-manager".to_vec(),
                    executable: true,
                },
            ],
            services: vec![],
        };

        // Python package
        let python_package = Package {
            metadata: PackageMetadata {
                name: "python".to_string(),
                version: "3.9.0".to_string(),
                description: "Python Interpreter".to_string(),
                arch: "x86_64".to_string(),
                size: 3 * 1024 * 1024, // 3MB
                dependencies: vec![],
            },
            files: vec![
                PackageFile {
                    path: "/bin/python".to_string(),
                    content: b"AIOS\x00\x10\x00\x00python-interpreter".to_vec(),
                    executable: true,
                },
                PackageFile {
                    path: "/bin/pip".to_string(),
                    content: b"AIOS\x00\x10\x00\x00pip-package-installer".to_vec(),
                    executable: true,
                },
            ],
            services: vec![],
        };

        // Store as available packages (not installed)
        self.repositories[0].packages.insert("claude-code".to_string(), claude_package.metadata.clone());
        self.repositories[0].packages.insert("nodejs".to_string(), nodejs_package.metadata.clone());
        self.repositories[0].packages.insert("python".to_string(), python_package.metadata.clone());
    }

    pub fn add_repository(&mut self, name: String, url: String) {
        let repo = Repository {
            name,
            url,
            packages: BTreeMap::new(),
        };
        self.repositories.push(repo);
    }

    pub fn install_package(&mut self, package_name: &str, filesystem: &mut MemoryFileSystem, system_manager: &mut SystemManager) -> Result<(), &'static str> {
        // Check if already installed
        if self.installed_packages.contains_key(package_name) {
            return Err("Package already installed");
        }

        // Find package in repositories
        let package = self.find_package(package_name)?;

        // Check dependencies
        for dep in &package.metadata.dependencies {
            if !self.is_package_installed(dep) {
                crate::println!("Installing dependency: {}", dep);
                self.install_package(dep, filesystem, system_manager)?;
            }
        }

        // Install files
        for file in &package.files {
            if file.executable {
                filesystem.create_executable(&file.path, &file.content)?;
            } else {
                filesystem.create_file(&file.path, &file.content)?;
            }
        }

        // Register services
        for service in &package.services {
            use crate::systemd::{ServiceUnit, ServiceState};
            let service_unit = ServiceUnit {
                name: service.name.clone(),
                description: format!("Service for {}", package.metadata.name),
                exec_start: service.command.clone(),
                working_directory: Some("/".to_string()),
                environment: vec![],
                container: if service.container { Some("default".to_string()) } else { None },
                state: ServiceState::Inactive,
            };
            
            system_manager.create_service(service_unit)?;
        }

        // Mark as installed
        self.installed_packages.insert(package_name.to_string(), package);

        crate::println!("Successfully installed package: {}", package_name);
        Ok(())
    }

    pub fn remove_package(&mut self, package_name: &str, filesystem: &mut MemoryFileSystem) -> Result<(), &'static str> {
        let package = self.installed_packages.get(package_name)
            .ok_or("Package not installed")?
            .clone();

        // Remove files
        for file in &package.files {
            let _ = filesystem.remove_file(&file.path); // Ignore errors for now
        }

        // Remove from installed list
        self.installed_packages.remove(package_name);

        crate::println!("Successfully removed package: {}", package_name);
        Ok(())
    }

    pub fn list_installed(&self) {
        crate::println!("Installed packages:");
        crate::println!("{:<20} {:<10} {:<30}", "NAME", "VERSION", "DESCRIPTION");
        crate::println!("{}", "-".repeat(60));
        
        for package in self.installed_packages.values() {
            crate::println!("{:<20} {:<10} {:<30}", 
                package.metadata.name, 
                package.metadata.version, 
                package.metadata.description
            );
        }
    }

    pub fn list_available(&self) {
        crate::println!("Available packages:");
        crate::println!("{:<20} {:<10} {:<30}", "NAME", "VERSION", "DESCRIPTION");
        crate::println!("{}", "-".repeat(60));
        
        for repo in &self.repositories {
            for package in repo.packages.values() {
                let status = if self.installed_packages.contains_key(&package.name) {
                    "[installed]"
                } else {
                    ""
                };
                crate::println!("{:<20} {:<10} {:<20} {}", 
                    package.name, 
                    package.version, 
                    package.description,
                    status
                );
            }
        }
    }

    pub fn search_package(&self, query: &str) -> Vec<&PackageMetadata> {
        let mut results = Vec::new();
        for repo in &self.repositories {
            for package in repo.packages.values() {
                if package.name.contains(query) || package.description.contains(query) {
                    results.push(package);
                }
            }
        }
        results
    }

    pub fn show_package_info(&self, package_name: &str) -> Result<(), &'static str> {
        if let Some(package) = self.installed_packages.get(package_name) {
            crate::println!("Package: {}", package.metadata.name);
            crate::println!("Version: {}", package.metadata.version);
            crate::println!("Description: {}", package.metadata.description);
            crate::println!("Architecture: {}", package.metadata.arch);
            crate::println!("Size: {} bytes", package.metadata.size);
            crate::println!("Status: Installed");
            
            if !package.metadata.dependencies.is_empty() {
                crate::println!("Dependencies:");
                for dep in &package.metadata.dependencies {
                    crate::println!("  - {}", dep);
                }
            }
            
            if !package.files.is_empty() {
                crate::println!("Files:");
                for file in &package.files {
                    crate::println!("  {}", file.path);
                }
            }
            
            Ok(())
        } else {
            // Check in repositories
            for repo in &self.repositories {
                if let Some(package) = repo.packages.get(package_name) {
                    crate::println!("Package: {}", package.name);
                    crate::println!("Version: {}", package.version);
                    crate::println!("Description: {}", package.description);
                    crate::println!("Architecture: {}", package.arch);
                    crate::println!("Size: {} bytes", package.size);
                    crate::println!("Status: Available");
                    
                    if !package.dependencies.is_empty() {
                        crate::println!("Dependencies:");
                        for dep in &package.dependencies {
                            crate::println!("  - {}", dep);
                        }
                    }
                    
                    return Ok(());
                }
            }
            Err("Package not found")
        }
    }

    fn find_package(&self, package_name: &str) -> Result<Package, &'static str> {
        // For now, return built-in packages
        match package_name {
            "claude-code" => {
                Ok(Package {
                    metadata: PackageMetadata {
                        name: "claude-code".to_string(),
                        version: "1.0.0".to_string(),
                        description: "AI Development Environment".to_string(),
                        arch: "x86_64".to_string(),
                        size: 1024 * 1024,
                        dependencies: vec!["nodejs".to_string()],
                    },
                    files: vec![
                        PackageFile {
                            path: "/bin/claude-code".to_string(),
                            content: b"AIOS\x00\x10\x00\x00claude-code-ai-assistant".to_vec(),
                            executable: true,
                        },
                    ],
                    services: vec![
                        ServiceConfig {
                            name: "claude-code".to_string(),
                            command: "/bin/claude-code --daemon".to_string(),
                            container: true,
                            auto_start: false,
                        },
                    ],
                })
            }
            "nodejs" => {
                Ok(Package {
                    metadata: PackageMetadata {
                        name: "nodejs".to_string(),
                        version: "18.0.0".to_string(),
                        description: "JavaScript Runtime".to_string(),
                        arch: "x86_64".to_string(),
                        size: 2 * 1024 * 1024,
                        dependencies: vec![],
                    },
                    files: vec![
                        PackageFile {
                            path: "/bin/node".to_string(),
                            content: b"AIOS\x00\x10\x00\x00nodejs-runtime".to_vec(),
                            executable: true,
                        },
                    ],
                    services: vec![],
                })
            }
            "python" => {
                Ok(Package {
                    metadata: PackageMetadata {
                        name: "python".to_string(),
                        version: "3.9.0".to_string(),
                        description: "Python Interpreter".to_string(),
                        arch: "x86_64".to_string(),
                        size: 3 * 1024 * 1024,
                        dependencies: vec![],
                    },
                    files: vec![
                        PackageFile {
                            path: "/bin/python".to_string(),
                            content: b"AIOS\x00\x10\x00\x00python-interpreter".to_vec(),
                            executable: true,
                        },
                    ],
                    services: vec![],
                })
            }
            _ => Err("Package not found")
        }
    }

    fn is_package_installed(&self, package_name: &str) -> bool {
        self.installed_packages.contains_key(package_name)
    }

    pub fn update_repositories(&mut self) -> Result<(), &'static str> {
        crate::println!("Updating package repositories...");
        // In a real implementation, this would fetch from remote repositories
        crate::println!("Repository update completed");
        Ok(())
    }
}