use alloc::{collections::BTreeMap, string::String, vec::Vec, string::ToString, format};
use core::fmt;

#[derive(Debug, Clone, Copy, PartialEq)]
pub enum ProcessState {
    Ready,
    Running,
    Waiting,
    Terminated,
}

#[derive(Debug, Clone, Copy)]
pub struct ProcessId(pub usize);

#[derive(Debug)]
pub struct Process {
    pub pid: ProcessId,
    pub name: String,
    pub state: ProcessState,
    pub entry_point: usize,
    pub stack_pointer: usize,
    pub memory: Vec<u8>,
    pub exit_code: Option<i32>,
}

pub struct ProcessManager {
    processes: BTreeMap<usize, Process>,
    next_pid: usize,
    current_process: Option<usize>,
}

impl ProcessManager {
    pub fn new() -> Self {
        Self {
            processes: BTreeMap::new(),
            next_pid: 1,
            current_process: None,
        }
    }

    pub fn create_process(&mut self, name: String, program: &[u8]) -> Result<ProcessId, &'static str> {
        let pid = ProcessId(self.next_pid);
        self.next_pid += 1;

        // Simple ELF-like parsing (simplified for demo)
        let entry_point = self.parse_program(program)?;
        
        let process = Process {
            pid,
            name,
            state: ProcessState::Ready,
            entry_point,
            stack_pointer: 0x8000, // Simple stack setup
            memory: program.to_vec(),
            exit_code: None,
        };

        self.processes.insert(pid.0, process);
        Ok(pid)
    }

    pub fn start_process(&mut self, pid: ProcessId) -> Result<(), &'static str> {
        if let Some(process) = self.processes.get_mut(&pid.0) {
            if process.state == ProcessState::Ready {
                process.state = ProcessState::Running;
                self.current_process = Some(pid.0);
                crate::println!("Starting process: {} (PID: {})", process.name, pid.0);
                
                // In a real OS, this would switch to the process context
                // For now, we'll simulate execution
                self.simulate_execution(pid)?;
                Ok(())
            } else {
                Err("Process not in ready state")
            }
        } else {
            Err("Process not found")
        }
    }

    pub fn terminate_process(&mut self, pid: ProcessId, exit_code: i32) -> Result<(), &'static str> {
        if let Some(process) = self.processes.get_mut(&pid.0) {
            process.state = ProcessState::Terminated;
            process.exit_code = Some(exit_code);
            
            if self.current_process == Some(pid.0) {
                self.current_process = None;
            }
            
            crate::println!("Process {} terminated with exit code {}", process.name, exit_code);
            Ok(())
        } else {
            Err("Process not found")
        }
    }

    pub fn list_processes(&self) -> Vec<&Process> {
        self.processes.values().collect()
    }

    pub fn get_process(&self, pid: ProcessId) -> Option<&Process> {
        self.processes.get(&pid.0)
    }

    pub fn kill_process(&mut self, pid: ProcessId) -> Result<(), &'static str> {
        self.terminate_process(pid, -1)
    }

    // Simple program parser - in reality this would be a full ELF parser
    fn parse_program(&self, program: &[u8]) -> Result<usize, &'static str> {
        if program.len() < 4 {
            return Err("Invalid program format");
        }

        // Check for simple executable magic
        if &program[0..4] == b"AIOS" {
            // AIOS native executable format
            if program.len() < 8 {
                return Err("Invalid AIOS executable");
            }
            
            // Entry point stored at offset 4-8
            let entry_point = u32::from_le_bytes([program[4], program[5], program[6], program[7]]) as usize;
            Ok(entry_point)
        } else if &program[0..4] == [0x7f, b'E', b'L', b'F'] {
            // Basic ELF detection
            crate::println!("ELF executable detected (simplified parsing)");
            Ok(0x1000) // Default entry point for ELF
        } else {
            // Treat as raw machine code
            Ok(0x0)
        }
    }

    // Simulate process execution (in a real OS this would involve context switching)
    fn simulate_execution(&mut self, pid: ProcessId) -> Result<(), &'static str> {
        if let Some(process) = self.processes.get(&pid.0) {
            crate::println!("Executing process: {}", process.name);
            
            // Check if it's a built-in command or script
            if process.name == "hello" {
                crate::println!("Hello from AIOS process!");
                self.terminate_process(pid, 0)?;
            } else if process.name == "claude-code" {
                crate::println!("Claude Code AI Assistant starting...");
                crate::println!("AI development environment initialized");
                crate::println!("Type 'ai help' for available commands");
                self.terminate_process(pid, 0)?;
            } else if process.name.starts_with("ai-") {
                crate::println!("AI service: {}", process.name);
                crate::println!("Connecting to Claude API...");
                crate::println!("AI service ready");
                self.terminate_process(pid, 0)?;
            } else {
                // Unknown program
                crate::println!("Unknown program format, terminating");
                self.terminate_process(pid, 1)?;
            }
            
            Ok(())
        } else {
            Err("Process not found")
        }
    }

    pub fn execute_command(&mut self, command: &str, args: &[&str]) -> Result<ProcessId, &'static str> {
        // Create a simple program based on command
        let program_name = if command == "claude-code" {
            "claude-code".to_string()
        } else if command.starts_with("ai") {
            format!("ai-{}", args.join("-"))
        } else {
            command.to_string()
        };

        // Create a simple executable
        let mut program = Vec::new();
        program.extend_from_slice(b"AIOS"); // Magic
        program.extend_from_slice(&(0x1000u32).to_le_bytes()); // Entry point
        program.extend_from_slice(command.as_bytes()); // Command data

        let pid = self.create_process(program_name, &program)?;
        self.start_process(pid)?;
        Ok(pid)
    }
}

impl fmt::Display for ProcessState {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self {
            ProcessState::Ready => write!(f, "Ready"),
            ProcessState::Running => write!(f, "Running"),
            ProcessState::Waiting => write!(f, "Waiting"),
            ProcessState::Terminated => write!(f, "Terminated"),
        }
    }
}

impl fmt::Display for ProcessId {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        write!(f, "{}", self.0)
    }
}