use alloc::{string::String, vec::Vec, vec, string::ToString};
use crate::{print, println};
use crate::systemd::SystemManager;
use crate::filesystem::MemoryFileSystem;
use crate::process::ProcessManager;
use crate::package::PackageManager;
use lazy_static::lazy_static;
use spin::Mutex;

lazy_static! {
    static ref SHELL: Mutex<Shell> = Mutex::new(Shell::new());
}

pub struct Shell {
    input_buffer: String,
    history: Vec<String>,
    prompt: &'static str,
    system_manager: SystemManager,
    filesystem: MemoryFileSystem,
    process_manager: ProcessManager,
    package_manager: PackageManager,
}

impl Shell {
    fn new() -> Self {
        // 一時的にメモリを使う部分を最小限に
        let filesystem = MemoryFileSystem::new();
        let process_manager = ProcessManager::new();
        let system_manager = SystemManager::new();
        let package_manager = PackageManager::new();
        
        Self {
            input_buffer: String::new(),
            history: Vec::new(),
            prompt: "aios$ ",
            system_manager,
            filesystem,
            process_manager,
            package_manager,
        }
    }

    fn execute_command(&mut self, command: &str) {
        let parts: Vec<&str> = command.trim().split_whitespace().collect();
        if parts.is_empty() {
            return;
        }

        match parts[0] {
            "help" => self.show_help(),
            "clear" => self.clear_screen(),
            "echo" => {
                for arg in &parts[1..] {
                    print!("{} ", arg);
                }
                println!();
            }
            "history" => self.show_history(),
            "systemctl" => {
                if let Err(e) = self.system_manager.systemctl(&parts[1..]) {
                    println!("systemctl: {}", e);
                }
            }
            "nspawn" => {
                if let Err(e) = self.system_manager.nspawn(&parts[1..]) {
                    println!("systemd-nspawn: {}", e);
                }
            }
            "claude" => self.execute_claude_command(&parts[1..]),
            "ai" => self.execute_ai_command(&parts[1..]),
            // File system commands
            "ls" => self.list_directory(&parts[1..]),
            "cd" => self.change_directory(&parts[1..]),
            "pwd" => println!("{}", self.filesystem.get_current_directory()),
            "mkdir" => self.make_directory(&parts[1..]),
            "cat" => self.cat_file(&parts[1..]),
            "touch" => self.create_file(&parts[1..]),
            "rm" => self.remove_file(&parts[1..]),
            // Process commands
            "ps" => self.list_processes(),
            "kill" => self.kill_process(&parts[1..]),
            "exec" => self.execute_program(&parts[1..]),
            // Package manager commands
            "pkg" => self.handle_package_command(&parts[1..]),
            "exit" => {
                println!("Goodbye!");
                // TODO: 適切なシャットダウン処理
            }
            _ => {
                // Try to execute as a program
                if let Err(e) = self.process_manager.execute_command(parts[0], &parts[1..]) {
                    println!("Unknown command: {}. Type 'help' for available commands.", parts[0]);
                    println!("Error: {}", e);
                }
            }
        }
    }

    fn execute_claude_command(&mut self, args: &[&str]) {
        if args.is_empty() {
            println!("Claude Code integration - AI-powered development environment");
            println!("Usage: claude <subcommand>");
            println!("  start    - Start Claude Code server");
            println!("  stop     - Stop Claude Code server");
            println!("  status   - Show Claude Code status");
            return;
        }

        match args[0] {
            "start" => {
                println!("Starting Claude Code in container...");
                // TODO: Claude Codeをコンテナ内で起動
                if let Err(e) = self.start_claude_service() {
                    println!("Failed to start Claude Code: {}", e);
                }
            }
            "stop" => {
                println!("Stopping Claude Code...");
                if let Err(e) = self.system_manager.stop_service("claude-code") {
                    println!("Failed to stop Claude Code: {}", e);
                }
            }
            "status" => {
                if let Err(e) = self.system_manager.systemctl(&["status", "claude-code"]) {
                    println!("Claude Code status: {}", e);
                }
            }
            _ => println!("Unknown claude subcommand: {}", args[0]),
        }
    }

    fn execute_ai_command(&mut self, args: &[&str]) {
        if args.is_empty() {
            println!("AI assistant integration");
            println!("Usage: ai <query>");
            return;
        }

        let query = args.join(" ");
        println!("AI Query: {}", query);
        println!("Response: This would integrate with Claude API for AI assistance");
        // TODO: 実際のAI統合
    }

    fn start_claude_service(&mut self) -> Result<(), &'static str> {
        use crate::systemd::{ServiceUnit, ServiceState};

        // Claude Codeサービスを作成
        let service = ServiceUnit {
            name: "claude-code".to_string(),
            description: "Claude Code AI Development Environment".to_string(),
            exec_start: "/usr/local/bin/claude-code".to_string(),
            working_directory: Some("/workspace".to_string()),
            environment: vec![
                ("ANTHROPIC_API_KEY".to_string(), "your-api-key".to_string()),
                ("PORT".to_string(), "3000".to_string()),
            ],
            container: Some("claude-container".to_string()),
            state: ServiceState::Inactive,
        };

        // コンテナを作成
        self.system_manager.create_container(
            "claude-container".to_string(),
            "/containers/claude".to_string(),
        )?;

        // サービスを登録・起動
        self.system_manager.create_service(service)?;
        self.system_manager.start_service("claude-code")
    }

    fn show_help(&self) {
        println!("AIOS Shell - Available Commands:");
        println!("  help        - Show this help message");
        println!("  clear       - Clear the screen");
        println!("  echo <text> - Print text to output");
        println!("  history     - Show command history");
        println!("");
        println!("File System:");
        println!("  ls [path]   - List directory contents");
        println!("  cd <path>   - Change directory");
        println!("  pwd         - Print working directory");
        println!("  mkdir <dir> - Create directory");
        println!("  cat <file>  - Display file contents");
        println!("  touch <file>- Create empty file");
        println!("  rm <file>   - Remove file");
        println!("");
        println!("Process Management:");
        println!("  ps          - List running processes");
        println!("  exec <prog> - Execute program");
        println!("  kill <pid>  - Kill process by PID");
        println!("");
        println!("Package Management:");
        println!("  pkg install <name>  - Install package");
        println!("  pkg remove <name>   - Remove package");
        println!("  pkg list            - List installed packages");
        println!("  pkg search <query>  - Search packages");
        println!("  pkg info <name>     - Show package info");
        println!("  pkg update          - Update repositories");
        println!("");
        println!("System:");
        println!("  systemctl   - Control system services");
        println!("  claude      - Claude Code integration");
        println!("  ai <query>  - AI assistant");
        println!("  exit        - Exit shell");
    }

    // File system command implementations
    fn list_directory(&mut self, args: &[&str]) {
        let path = if args.is_empty() {
            self.filesystem.get_current_directory()
        } else {
            args[0]
        };

        match self.filesystem.list_directory(path) {
            Ok(entries) => {
                for entry in entries {
                    println!("{} {:>8} {}", entry.file_type, entry.size, entry.name);
                }
            }
            Err(e) => println!("ls: {}", e),
        }
    }

    fn change_directory(&mut self, args: &[&str]) {
        if args.is_empty() {
            if let Err(e) = self.filesystem.change_directory("/") {
                println!("cd: {}", e);
            }
        } else {
            if let Err(e) = self.filesystem.change_directory(args[0]) {
                println!("cd: {}", e);
            }
        }
    }

    fn make_directory(&mut self, args: &[&str]) {
        if args.is_empty() {
            println!("mkdir: missing directory name");
            return;
        }

        if let Err(e) = self.filesystem.create_directory(args[0]) {
            println!("mkdir: {}", e);
        }
    }

    fn cat_file(&mut self, args: &[&str]) {
        if args.is_empty() {
            println!("cat: missing file name");
            return;
        }

        match self.filesystem.read_file(args[0]) {
            Ok(data) => {
                if let Ok(text) = core::str::from_utf8(data) {
                    print!("{}", text);
                } else {
                    println!("cat: binary file");
                }
            }
            Err(e) => println!("cat: {}", e),
        }
    }

    fn create_file(&mut self, args: &[&str]) {
        if args.is_empty() {
            println!("touch: missing file name");
            return;
        }

        if let Err(e) = self.filesystem.create_file(args[0], b"") {
            println!("touch: {}", e);
        }
    }

    fn remove_file(&mut self, args: &[&str]) {
        if args.is_empty() {
            println!("rm: missing file name");
            return;
        }

        if let Err(e) = self.filesystem.remove_file(args[0]) {
            println!("rm: {}", e);
        }
    }

    // Process command implementations
    fn list_processes(&mut self) {
        let processes = self.process_manager.list_processes();
        println!("PID    NAME           STATE");
        for process in processes {
            println!("{:<6} {:<14} {}", process.pid, process.name, process.state);
        }
    }

    fn kill_process(&mut self, args: &[&str]) {
        if args.is_empty() {
            println!("kill: missing process ID");
            return;
        }

        if let Ok(pid) = args[0].parse::<usize>() {
            if let Err(e) = self.process_manager.kill_process(crate::process::ProcessId(pid)) {
                println!("kill: {}", e);
            }
        } else {
            println!("kill: invalid process ID");
        }
    }

    fn execute_program(&mut self, args: &[&str]) {
        if args.is_empty() {
            println!("exec: missing program name");
            return;
        }

        if let Err(e) = self.process_manager.execute_command(args[0], &args[1..]) {
            println!("exec: {}", e);
        }
    }

    // Package manager command implementations
    fn handle_package_command(&mut self, args: &[&str]) {
        if args.is_empty() {
            println!("pkg: missing command");
            println!("Usage: pkg <command> [args...]");
            println!("Commands: install, remove, list, search, info, update");
            return;
        }

        match args[0] {
            "install" => {
                if args.len() < 2 {
                    println!("pkg install: missing package name");
                    return;
                }
                self.install_package(args[1]);
            }
            "remove" => {
                if args.len() < 2 {
                    println!("pkg remove: missing package name");
                    return;
                }
                self.remove_package(args[1]);
            }
            "list" => {
                if args.len() > 1 && args[1] == "available" {
                    self.package_manager.list_available();
                } else {
                    self.package_manager.list_installed();
                }
            }
            "search" => {
                if args.len() < 2 {
                    println!("pkg search: missing search query");
                    return;
                }
                self.search_packages(args[1]);
            }
            "info" => {
                if args.len() < 2 {
                    println!("pkg info: missing package name");
                    return;
                }
                if let Err(e) = self.package_manager.show_package_info(args[1]) {
                    println!("pkg info: {}", e);
                }
            }
            "update" => {
                if let Err(e) = self.package_manager.update_repositories() {
                    println!("pkg update: {}", e);
                }
            }
            _ => {
                println!("pkg: unknown command '{}'", args[0]);
                println!("Available commands: install, remove, list, search, info, update");
            }
        }
    }

    fn install_package(&mut self, package_name: &str) {
        println!("Installing package: {}", package_name);
        match self.package_manager.install_package(package_name, &mut self.filesystem, &mut self.system_manager) {
            Ok(()) => {
                println!("Package '{}' installed successfully", package_name);
                
                // Special handling for Claude Code
                if package_name == "claude-code" {
                    println!("Claude Code is now available!");
                    println!("You can start it with: claude start");
                    println!("Or run it directly: /bin/claude-code");
                }
            }
            Err(e) => println!("Failed to install package '{}': {}", package_name, e),
        }
    }

    fn remove_package(&mut self, package_name: &str) {
        println!("Removing package: {}", package_name);
        match self.package_manager.remove_package(package_name, &mut self.filesystem) {
            Ok(()) => println!("Package '{}' removed successfully", package_name),
            Err(e) => println!("Failed to remove package '{}': {}", package_name, e),
        }
    }

    fn search_packages(&mut self, query: &str) {
        let results = self.package_manager.search_package(query);
        if results.is_empty() {
            println!("No packages found matching '{}'", query);
        } else {
            println!("Packages matching '{}':", query);
            println!("{:<20} {:<10} {:<30}", "NAME", "VERSION", "DESCRIPTION");
            println!("{}", "-".repeat(60));
            for package in results {
                println!("{:<20} {:<10} {:<30}", package.name, package.version, package.description);
            }
        }
    }

    fn clear_screen(&self) {
        // VGAバッファをクリア
        crate::vga_buffer::WRITER.lock().clear_screen();
    }

    fn show_history(&self) {
        println!("Command History:");
        for (i, cmd) in self.history.iter().enumerate() {
            println!("  {}: {}", i + 1, cmd);
        }
    }

    fn show_prompt(&self) {
        print!("{}", self.prompt);
    }

    fn add_to_history(&mut self, command: String) {
        if !command.trim().is_empty() {
            self.history.push(command);
        }
    }
}

// VGAバッファにclear_screen機能を追加
impl crate::vga_buffer::Writer {
    pub fn clear_screen(&mut self) {
        use crate::vga_buffer::{BUFFER_HEIGHT, BUFFER_WIDTH, ColorCode, Color, ScreenChar};
        
        let blank = ScreenChar {
            ascii_character: b' ',
            color_code: ColorCode::new(Color::Yellow, Color::Black),
        };
        
        for row in 0..BUFFER_HEIGHT {
            for col in 0..BUFFER_WIDTH {
                self.buffer.chars[row][col].write(blank);
            }
        }
        self.column_position = 0;
    }
}

pub fn handle_keyboard_input(character: char) {
    let mut shell = SHELL.lock();
    match character {
        '\n' => {
            println!();
            let command = shell.input_buffer.clone();
            shell.add_to_history(command.clone());
            shell.execute_command(&command);
            shell.input_buffer.clear();
            shell.show_prompt();
        }
        '\x08' => { // Backspace
            if !shell.input_buffer.is_empty() {
                shell.input_buffer.pop();
                print!("\x08 \x08"); // Backspace, space, backspace
            }
        }
        c if c.is_ascii() && !c.is_control() => {
            shell.input_buffer.push(c);
            print!("{}", c);
        }
        _ => {} // 他の文字は無視
    }
}

pub fn init() {
    println!("AIOS Shell v0.1.0 - AI-Integrated Operating System");
    println!("Type 'help' for available commands.");
    SHELL.lock().show_prompt();
}