use alloc::{collections::BTreeMap, string::String, vec::Vec};
use core::fmt;

#[derive(Debug, Clone, Copy, PartialEq)]
pub enum ServiceState {
    Inactive,
    Active,
    Failed,
    Activating,
    Deactivating,
}

#[derive(Debug, Clone)]
pub struct ServiceUnit {
    pub name: String,
    pub description: String,
    pub exec_start: String,
    pub working_directory: Option<String>,
    pub environment: Vec<(String, String)>,
    pub container: Option<String>,
    pub state: ServiceState,
}

#[derive(Debug, Clone)]
pub struct Container {
    pub name: String,
    pub root_directory: String,
    pub private_network: bool,
    pub bind_mounts: Vec<(String, String)>,
    pub capability_bounding_set: Vec<String>,
}

pub struct SystemManager {
    services: BTreeMap<String, ServiceUnit>,
    containers: BTreeMap<String, Container>,
}

impl SystemManager {
    pub fn new() -> Self {
        Self {
            services: BTreeMap::new(),
            containers: BTreeMap::new(),
        }
    }

    pub fn create_container(&mut self, name: String, root_dir: String) -> Result<(), &'static str> {
        let container = Container {
            name: name.clone(),
            root_directory: root_dir,
            private_network: true,
            bind_mounts: Vec::new(),
            capability_bounding_set: Vec::new(),
        };
        
        self.containers.insert(name, container);
        Ok(())
    }

    pub fn create_service(&mut self, service: ServiceUnit) -> Result<(), &'static str> {
        self.services.insert(service.name.clone(), service);
        Ok(())
    }

    pub fn start_service(&mut self, name: &str) -> Result<(), &'static str> {
        if let Some(service) = self.services.get_mut(name) {
            service.state = ServiceState::Activating;
            
            // コンテナ内で実行する場合
            if let Some(container_name) = &service.container {
                if let Some(_container) = self.containers.get(container_name) {
                    // TODO: nspawn風のコンテナ内でプロセス実行
                    crate::println!("Starting service {} in container {}", name, container_name);
                } else {
                    return Err("Container not found");
                }
            } else {
                // ホスト上で直接実行
                crate::println!("Starting service {} on host", name);
            }
            
            service.state = ServiceState::Active;
            Ok(())
        } else {
            Err("Service not found")
        }
    }

    pub fn stop_service(&mut self, name: &str) -> Result<(), &'static str> {
        if let Some(service) = self.services.get_mut(name) {
            service.state = ServiceState::Deactivating;
            crate::println!("Stopping service {}", name);
            service.state = ServiceState::Inactive;
            Ok(())
        } else {
            Err("Service not found")
        }
    }

    pub fn list_services(&self) {
        crate::println!("UNIT                     LOAD   ACTIVE SUB     DESCRIPTION");
        for (name, service) in &self.services {
            crate::println!("{:<24} loaded {:?} running {}", 
                name, service.state, service.description);
        }
    }

    pub fn list_containers(&self) {
        crate::println!("MACHINE CLASS     SERVICE");
        for (name, _container) in &self.containers {
            crate::println!("{:<8} container -", name);
        }
    }

    // systemctl風のコマンド処理
    pub fn systemctl(&mut self, args: &[&str]) -> Result<(), &'static str> {
        if args.len() < 2 {
            return Err("Usage: systemctl <command> <service>");
        }

        match args[0] {
            "start" => self.start_service(args[1]),
            "stop" => self.stop_service(args[1]),
            "status" => {
                if let Some(service) = self.services.get(args[1]) {
                    crate::println!("● {}", service.name);
                    crate::println!("   Loaded: loaded");
                    crate::println!("   Active: {:?}", service.state);
                    crate::println!("   Description: {}", service.description);
                    Ok(())
                } else {
                    Err("Service not found")
                }
            }
            "list-units" => {
                self.list_services();
                Ok(())
            }
            _ => Err("Unknown systemctl command"),
        }
    }

    // systemd-nspawn風のコマンド
    pub fn nspawn(&mut self, args: &[&str]) -> Result<(), &'static str> {
        if args.len() < 2 {
            return Err("Usage: nspawn --directory=<dir> <command>");
        }

        let mut directory = None;
        let mut command = None;

        for arg in args {
            if arg.starts_with("--directory=") {
                directory = Some(&arg[12..]);
            } else if !arg.starts_with("--") {
                command = Some(*arg);
                break;
            }
        }

        if let (Some(dir), Some(cmd)) = (directory, command) {
            crate::println!("Starting container in {} executing {}", dir, cmd);
            // TODO: 実際のコンテナ実行
            Ok(())
        } else {
            Err("Invalid nspawn arguments")
        }
    }
}

impl fmt::Display for ServiceState {
    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
        match self {
            ServiceState::Inactive => write!(f, "inactive"),
            ServiceState::Active => write!(f, "active"),
            ServiceState::Failed => write!(f, "failed"),
            ServiceState::Activating => write!(f, "activating"),
            ServiceState::Deactivating => write!(f, "deactivating"),
        }
    }
}